*** a/src/backend/access/brin/brin_pageops.c
--- b/src/backend/access/brin/brin_pageops.c
***************
*** 771,776 **** brin_getinsertbuffer(Relation irel, Buffer oldbuf, Size itemsz,
--- 771,781 ----
  			UnlockRelationForExtension(irel, ExclusiveLock);
  
  		page = BufferGetPage(buf);
+ 		if (PageIsNew(page))
+ 		{
+ 			MarkBufferDirty(buf);
+ 			PageInit(page, BufferGetPageSize(buf), 0);
+ 		}
  
  		/*
  		 * We have a new buffer to insert into.  Check that the new page has
*** a/src/backend/access/heap/hio.c
--- b/src/backend/access/heap/hio.c
***************
*** 393,398 **** RelationGetBufferForTuple(Relation relation, Size len,
--- 393,404 ----
  		 * we're done.
  		 */
  		page = BufferGetPage(buffer);
+ 		if (PageIsNew(page))
+ 		{
+ 			MarkBufferDirty(buffer);
+ 			PageInit(page, BufferGetPageSize(buffer), 0);
+ 		}
+ 
  		pageFreeSpace = PageGetHeapFreeSpace(page);
  		if (len + saveFreeSpace <= pageFreeSpace)
  		{
*** a/src/backend/storage/buffer/bufmgr.c
--- b/src/backend/storage/buffer/bufmgr.c
***************
*** 90,95 **** int			effective_io_concurrency = 0;
--- 90,96 ----
   * effective_io_concurrency parameter set.
   */
  int			target_prefetch_pages = 0;
+ int			extend_num_pages = 0;
  
  /* local state for StartBufferIO and related functions */
  static BufferDesc *InProgressBuf = NULL;
***************
*** 394,400 **** ForgetPrivateRefCountEntry(PrivateRefCountEntry *ref)
  static Buffer ReadBuffer_common(SMgrRelation reln, char relpersistence,
  				  ForkNumber forkNum, BlockNumber blockNum,
  				  ReadBufferMode mode, BufferAccessStrategy strategy,
! 				  bool *hit);
  static bool PinBuffer(BufferDesc *buf, BufferAccessStrategy strategy);
  static void PinBuffer_Locked(BufferDesc *buf);
  static void UnpinBuffer(BufferDesc *buf, bool fixOwner);
--- 395,401 ----
  static Buffer ReadBuffer_common(SMgrRelation reln, char relpersistence,
  				  ForkNumber forkNum, BlockNumber blockNum,
  				  ReadBufferMode mode, BufferAccessStrategy strategy,
! 				  bool *hit, Relation rel);
  static bool PinBuffer(BufferDesc *buf, BufferAccessStrategy strategy);
  static void PinBuffer_Locked(BufferDesc *buf);
  static void UnpinBuffer(BufferDesc *buf, bool fixOwner);
***************
*** 621,627 **** ReadBufferExtended(Relation reln, ForkNumber forkNum, BlockNumber blockNum,
  	 */
  	pgstat_count_buffer_read(reln);
  	buf = ReadBuffer_common(reln->rd_smgr, reln->rd_rel->relpersistence,
! 							forkNum, blockNum, mode, strategy, &hit);
  	if (hit)
  		pgstat_count_buffer_hit(reln);
  	return buf;
--- 622,628 ----
  	 */
  	pgstat_count_buffer_read(reln);
  	buf = ReadBuffer_common(reln->rd_smgr, reln->rd_rel->relpersistence,
! 							forkNum, blockNum, mode, strategy, &hit, reln);
  	if (hit)
  		pgstat_count_buffer_hit(reln);
  	return buf;
***************
*** 649,655 **** ReadBufferWithoutRelcache(RelFileNode rnode, ForkNumber forkNum,
  	Assert(InRecovery);
  
  	return ReadBuffer_common(smgr, RELPERSISTENCE_PERMANENT, forkNum, blockNum,
! 							 mode, strategy, &hit);
  }
  
  
--- 650,656 ----
  	Assert(InRecovery);
  
  	return ReadBuffer_common(smgr, RELPERSISTENCE_PERMANENT, forkNum, blockNum,
! 							 mode, strategy, &hit, NULL);
  }
  
  
***************
*** 661,667 **** ReadBufferWithoutRelcache(RelFileNode rnode, ForkNumber forkNum,
  static Buffer
  ReadBuffer_common(SMgrRelation smgr, char relpersistence, ForkNumber forkNum,
  				  BlockNumber blockNum, ReadBufferMode mode,
! 				  BufferAccessStrategy strategy, bool *hit)
  {
  	BufferDesc *bufHdr;
  	Block		bufBlock;
--- 662,668 ----
  static Buffer
  ReadBuffer_common(SMgrRelation smgr, char relpersistence, ForkNumber forkNum,
  				  BlockNumber blockNum, ReadBufferMode mode,
! 				  BufferAccessStrategy strategy, bool *hit, Relation rel)
  {
  	BufferDesc *bufHdr;
  	Block		bufBlock;
***************
*** 685,691 **** ReadBuffer_common(SMgrRelation smgr, char relpersistence, ForkNumber forkNum,
--- 686,695 ----
  
  	/* Substitute proper block number if caller asked for P_NEW */
  	if (isExtend)
+ 	{
  		blockNum = smgrnblocks(smgr, forkNum);
+ 		//blockNum += extend_num_pages;
+ 	}
  
  	if (isLocalBuf)
  	{
***************
*** 814,823 **** ReadBuffer_common(SMgrRelation smgr, char relpersistence, ForkNumber forkNum,
--- 818,836 ----
  
  	if (isExtend)
  	{
+ 		int blkCount = 0;
+ 
  		/* new buffers are zero-filled */
  		MemSet((char *) bufBlock, 0, BLCKSZ);
  		/* don't set checksum for all-zero page */
  		smgrextend(smgr, forkNum, blockNum, (char *) bufBlock, false);
+ 
+ 		while (blkCount < extend_num_pages)
+ 		{
+ 			blkCount++;
+ 			smgrextend(smgr, forkNum, blockNum+blkCount, (char *) bufBlock, false);
+ 			RecordPageWithFreeSpace(rel, blockNum+blkCount, 8126);
+ 		}
  	}
  	else
  	{
*** a/src/backend/utils/misc/guc.c
--- b/src/backend/utils/misc/guc.c
***************
*** 2683,2688 **** static struct config_int ConfigureNamesInt[] =
--- 2683,2698 ----
  		NULL, NULL, NULL
  	},
  
+ 	{
+ 		{"extend_num_pages", PGC_SUSET, RESOURCES_ASYNCHRONOUS,
+ 			gettext_noop("Sets the Number of pages to extended at one time."),
+ 			NULL
+ 		},
+ 		&extend_num_pages,
+ 		0, 0, 100,
+ 		NULL, NULL, NULL
+ 	},
+ 
  	/* End-of-list marker */
  	{
  		{NULL, 0, 0, NULL, NULL}, NULL, 0, 0, 0, NULL, NULL, NULL
*** a/src/backend/utils/misc/postgresql.conf.sample
--- b/src/backend/utils/misc/postgresql.conf.sample
***************
*** 139,144 ****
--- 139,146 ----
  
  #temp_file_limit = -1			# limits per-session temp file space
  					# in kB, or -1 for no limit
+ #extend_num_pages = 0			# number of extra pages allocate during extend
+ 					# min 0 max 100 pages
  
  # - Kernel Resource Usage -
  
*** a/src/include/storage/bufmgr.h
--- b/src/include/storage/bufmgr.h
***************
*** 60,65 **** extern PGDLLIMPORT char *BufferBlocks;
--- 60,66 ----
  
  /* in guc.c */
  extern int	effective_io_concurrency;
+ extern int	extend_num_pages;
  
  /* in localbuf.c */
  extern PGDLLIMPORT int NLocBuffer;
